@@ -23,7 +23,6 @@ use itertools::Itertools;
 use rayon::prelude::*;
 use serde::*;
 use smallvec::SmallVec;
-use sysinfo::DiskType;
 use thread_local::ThreadLocal;
 
 use crate::arg::Arg;
@@ -607,8 +606,8 @@ where
             file_hash: hash,
             files: files.to_vec(),
         })
-        .filter(group_post_filter)
         .chain(groups_to_pass.into_iter())
+        .filter(group_post_filter)
         .collect()
 }
 
@@ -799,7 +798,7 @@ fn update_file_locations(ctx: &GroupCtx<'_>, groups: &mut (impl FileCollection +
     let err_counters = atomic_counter_vec(ctx.devices.len());
     groups.for_each_mut(|fi| {
         let device: &DiskDevice = &ctx.devices[fi.get_device_index()];
-        if device.disk_type != DiskType::SSD {
+        if device.disk_type != sysinfo::DiskType::SSD {
             if let Err(e) = fi.fetch_physical_location() {
                 handle_fetch_physical_location_err(ctx, &err_counters, fi, e)
             }
@@ -1227,6 +1226,7 @@ mod test {
     use std::sync::Mutex;
 
     use rand::seq::SliceRandom;
+    use sysinfo::DiskType;
 
     use crate::path::Path;
     use crate::util::test::*;
@@ -1790,6 +1790,28 @@ mod test {
         })
     }
 
+    #[test]
+    fn unique_files() {
+        with_dir("main/unique_files", |root| {
+            let file1 = root.join("file1");
+            let file2 = root.join("file2");
+            let file3 = root.join("file3");
+            write_test_file(&file1, b"duplicate", b"", b"");
+            write_test_file(&file2, b"duplicate", b"", b"");
+            write_test_file(&file3, b"unique", b"", b"");
+
+            let file3 = Path::from(file3);
+
+            let log = test_log();
+            let mut config = GroupConfig::default();
+            config.unique = true;
+            config.paths = vec![file1.into(), file2.into(), file3.clone()];
+            let results = group_files(&config, &log).unwrap();
+            assert_eq!(results.len(), 1);
+            assert_eq!(results[0].files[0].path, file3);
+        });
+    }
+
     #[test]
     fn report() {
         with_dir("main/report", |root| {
@@ -1826,7 +1848,7 @@ mod test {
                     device: 0,
                 },
                 len: FileLen(1024),
-                location: id * 1024,
+                location: id as u64 * 1024,
             }
         }
 
